@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# atlases.py - API which provides access to the atlas image files contained 
+# atlases.py - API which provides access to the atlas image files contained
 #              in $FSLDIR/data/atlases/
 #
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
@@ -40,6 +40,8 @@ load an atlas image, which will be one of the following atlas-specific
 """
 
 
+from __future__ import division
+
 import xml.etree.ElementTree              as et
 import os.path                            as op
 import                                       glob
@@ -68,7 +70,7 @@ class AtlasRegistry(notifier.Notifier):
     ``$FSLDIR/data/atlases``, and builds a list of :class:`AtlasDescription`
     instances, each of which contains information about one atlas.
 
-    
+
     The :meth:`addAtlas` method allows other atlases to be added to the
     registry. Whenever a new atlas is added, the ``AtlasRegistry`` notifies
     any registered listeners via the :class:`.Notifier` interface with the
@@ -81,18 +83,18 @@ class AtlasRegistry(notifier.Notifier):
 
     The ``AtlasRegistry`` stores a list of all known atlases via the
     :mod:`.settings` module. When an ``AtlasRegistry`` is created, it loads
-    in any previously known atlases. Whenever a new atlas is added, this 
+    in any previously known atlases. Whenever a new atlas is added, this
     list is updated. See the :meth:`__getKnownAtlases` and
     :meth:`_saveKnownAtlases` methods.
     """
 
-    
+
     def __init__(self):
         """Create an ``AtlasRegistry``. """
 
         # A list of all AtlasDescription
-        # instances in existence, sorted 
-        # by AtlasDescription.name. 
+        # instances in existence, sorted
+        # by AtlasDescription.name.
         self.__atlasDescs = []
 
 
@@ -105,9 +107,9 @@ class AtlasRegistry(notifier.Notifier):
         log.debug('Initialising atlas registry')
         self.__atlasDescs = []
 
-        # Get $FSLDIR atlases 
+        # Get $FSLDIR atlases
         fslPaths = []
-        if platform.fsldir is not None: 
+        if platform.fsldir is not None:
             fsldir   = op.join(platform.fsldir, 'data', 'atlases')
             fslPaths = sorted(glob.glob(op.join(fsldir, '*.xml')))
 
@@ -115,14 +117,14 @@ class AtlasRegistry(notifier.Notifier):
         # been loaded in the past
         extraIDs, extraPaths = self.__getKnownAtlases()
 
-        # FSLDIR atlases first, any 
+        # FSLDIR atlases first, any
         # other atlases second.
         atlasPaths = list(fslPaths)         + extraPaths
         atlasIDs   = [None] * len(fslPaths) + extraIDs
 
         with self.skipAll():
             for atlasID, atlasPath in zip(atlasIDs, atlasPaths):
-                
+
                 # The FSLDIR atlases are probably
                 # listed twice - from the above glob,
                 # and from the saved extraPaths. So
@@ -137,7 +139,7 @@ class AtlasRegistry(notifier.Notifier):
                                 'specification {}'.format(atlasPath),
                                 exc_info=True)
 
-    
+
     def listAtlases(self):
         """Returns a list containing :class:`AtlasDescription` objects for
         all available atlases. The atlases are ordered in terms of the
@@ -157,17 +159,17 @@ class AtlasRegistry(notifier.Notifier):
         """Returns an :class:`AtlasDescription` instance describing the
         atlas with the given ``atlasID``.
         """
-        
+
         for desc in self.__atlasDescs:
             if desc.atlasID == atlasID:
                 return desc
-            
+
         raise KeyError('Unknown atlas ID: {}'.format(atlasID))
 
 
     def loadAtlas(self, atlasID, loadSummary=False, resolution=None):
         """Loads and returns an :class:`Atlas` instance for the atlas
-        with the given  ``atlasID``. 
+        with the given  ``atlasID``.
 
         :arg loadSummary: If ``True``, a 3D :class:`LabelAtlas` image is
                           loaded. Otherwise, if the atlas is probabilistic,
@@ -203,8 +205,8 @@ class AtlasRegistry(notifier.Notifier):
                        atlas with the given ID already exists, this new atlas
                        is given a unique id.
 
-        :arg save:     If ``True`` (the default), this atlas will be saved 
-                       so that it will be available in future instantiations. 
+        :arg save:     If ``True`` (the default), this atlas will be saved
+                       so that it will be available in future instantiations.
         """
 
         filename = op.abspath(filename)
@@ -233,7 +235,7 @@ class AtlasRegistry(notifier.Notifier):
             self.__saveKnownAtlases()
 
         self.notify(topic='add', value=desc)
-            
+
         return desc
 
 
@@ -248,15 +250,15 @@ class AtlasRegistry(notifier.Notifier):
                 log.debug('Removing atlas from registry: {} / {}'.format(
                     desc.atlasID,
                     desc.specPath))
-                
+
                 self.__atlasDescs.pop(i)
                 break
-        
+
         self.__saveKnownAtlases()
 
         self.notify(topic='remove', value=desc)
 
-    
+
     def __getKnownAtlases(self):
         """Returns a list of tuples containing the IDs and paths of all known
         atlases .
@@ -282,15 +284,15 @@ class AtlasRegistry(notifier.Notifier):
             paths = [e[1] for e in atlases]
 
             return names, paths
-            
+
         except:
             return [], []
 
-    
+
     def __saveKnownAtlases(self):
         """Saves the IDs and paths of all atlases which are currently in
         the registry. The atlases are saved via the :mod:`.settings` module.
-        """ 
+        """
 
         if self.__atlasDescs is None:
             return
@@ -302,10 +304,62 @@ class AtlasRegistry(notifier.Notifier):
 
         atlases = ['{}={}'.format(name, path) for name, path in atlases]
         atlases = op.pathsep.join(atlases)
-        
+
         fslsettings.write('fsl.data.atlases', atlases)
 
-    
+
+class AtlasLabel(object):
+    """The ``AtlasLabel`` class is used by the :class:`AtlasDescription` class
+    as a container object used for storing atlas label information.
+
+    An ``AtlasLabel`` instance contains the following attributes:
+
+    ========= ==============================================================
+    ``name``  Region name
+    ``index`` For probabilistic atlases, the volume index into the 4D atlas
+              image that corresponds to this region. For label atlases, the
+              value of voxels that are in this region. For summary images of
+              probabilistic atlases, add 1 to this value to get the
+              corresponding voxel values.
+    ``x``     X coordinate of the region in world space
+    ``y``     Y coordinate of the region in world space
+    ``z``     Z coordinate of the region in world space
+    ========= ==============================================================
+
+    .. note:: The ``x``, ``y`` and ``z`` label coordinates are pre-calculated
+              centre-of-gravity coordinates, as listed in the atlas xml file.
+              They are in the coordinate system defined by the transformation
+              matrix for the first image in the ``images`` list of the atlas
+              XML file (typically MNI152 space).
+    """
+
+    def __init__(self, name, index, x, y, z):
+        self.name  = name
+        self.index = index
+        self.x     = x
+        self.y     = y
+        self.z     = z
+
+
+    def __eq__(self, other):
+        """Compares the ``index`` of this ``AtlasLabel`` with another.
+        """
+        return self.index == other.index
+
+
+    def __neq__(self, other):
+        """Compares the ``index`` of this ``AtlasLabel`` with another.
+        """
+        return self.index != other.index
+
+
+    def __lt__(self, other):
+        """Compares this ``AtlasLabel`` with another by their ``index``
+        attribute.
+        """
+        return self.index < other.index
+
+
 class AtlasDescription(object):
     """An ``AtlasDescription`` instance parses and stores the information
     stored in the FSL XML file that describes a single FSL atlas.  An XML
@@ -325,11 +379,11 @@ class AtlasDescription(object):
                                 # label, Otherwise, if type is
                                 # Label, path to 3D label file
                                 # (identical to the summaryimagefile
-                                # below). The path must be specified 
+                                # below). The path must be specified
                                 # as relative to the location of this
                                 # XML file.
 
-            <summaryimagefile>  # Path to 3D summary file, with each 
+            <summaryimagefile>  # Path to 3D summary file, with each
             </summaryimagefile> # region having value (index + 1)
 
            </images>
@@ -342,7 +396,7 @@ class AtlasDescription(object):
          # index - For probabilistic atlases, index of corresponding volume in
          #         4D image file. For label images, the value of voxels which
          #         are in the corresponding region.
-         # 
+         #
          # x    |
          # y    |- XYZ *voxel* coordinates into the first image of the <images>
          #      |  list
@@ -352,7 +406,7 @@ class AtlasDescription(object):
         </data>
        </atlas>
 
-    
+
     Each ``AtlasDescription`` is assigned an identifier, which is simply the
     XML file name describing the atlas, sans-suffix, and converted to lower
     case.  For exmaple, the atlas described by:
@@ -363,7 +417,7 @@ class AtlasDescription(object):
 
         ``harvardoxford-cortical``
 
-    
+
     This identifier is intended to be unique.
 
 
@@ -371,53 +425,32 @@ class AtlasDescription(object):
 
     ================= ======================================================
     ``atlasID``       The atlas ID, as described above.
-    
+
     ``name``          Name of the atlas.
 
     ``specPath``      Path to the atlas XML specification file.
-    
+
     ``atlasType``     Atlas type - either *probabilistic* or *label*.
-    
+
     ``images``        A list of images available for this atlas - usually
                       :math:`1mm^3` and :math:`2mm^3` images are present.
-    
+
     ``summaryImages`` For probabilistic atlases, a list of *summary* images,
                       which are just 3D labelled variants of the atlas.
-    
+
     ``pixdims``       A list of ``(x, y, z)`` pixdim tuples in mm, one for
                       each image in ``images``.
 
     ``xforms``        A list of affine transformation matrices (as ``4*4``
                       ``numpy`` arrays), one for each image in ``images``,
                       defining the voxel to world coordinate transformations.
-    
-    ``labels``        A list of ``AtlasLabel`` objects, describing each
+
+    ``labels``        A list of :class`AtlasLabel` objects, describing each
                       region / label in the atlas.
     ================= ======================================================
-
-    Each ``AtlasLabel`` instance in the ``labels`` list contains the
-    following attributes:
-
-    ========= ==============================================================
-    ``name``  Region name
-    ``index`` For probabilistic atlases, the volume index into the 4D atlas
-              image that corresponds to this region. For label atlases, the
-              value of voxels that are in this region. For summary images of
-              probabilistic atlases, add 1 to this value to get the
-              corresponding voxel values.
-    ``x``     X coordinate of the region in world space
-    ``y``     Y coordinate of the region in world space
-    ``z``     Z coordinate of the region in world space
-    ========= ==============================================================
-
-    .. note:: The ``x``, ``y`` and ``z`` label coordinates are pre-calculated
-              centre-of-gravity coordinates, as listed in the atlas xml file.
-              They are in the coordinate system defined by the transformation
-              matrix for the first image in the ``images`` list.(typically
-              MNI152 space).
     """
 
-    
+
     def __init__(self, filename, atlasID=None):
         """Create an ``AtlasDescription`` instance.
 
@@ -440,7 +473,7 @@ class AtlasDescription(object):
         self.specPath  = op.abspath(filename)
         self.name      = header.find('name').text
         self.atlasType = header.find('type').text.lower()
- 
+
         # Spelling error in some of the atlas.xml files.
         if self.atlasType == 'probabalistic':
             self.atlasType = 'probabilistic'
@@ -452,7 +485,7 @@ class AtlasDescription(object):
         self.xforms        = []
 
         atlasDir = op.dirname(self.specPath)
-        
+
         for image in images:
             imagefile        = image.find('imagefile')       .text
             summaryimagefile = image.find('summaryimagefile').text
@@ -469,11 +502,6 @@ class AtlasDescription(object):
             self.pixdims      .append(i.pixdim[:3])
             self.xforms       .append(i.voxToWorldMat)
 
-        # A container object used for
-        # storing atlas label information
-        class AtlasLabel(object):
-            pass
-
         labels      = data.findall('label')
         self.labels = []
 
@@ -485,15 +513,15 @@ class AtlasDescription(object):
         coords = np.zeros((len(labels), 3), dtype=np.float32)
 
         for i, label in enumerate(labels):
-            
-            al        = AtlasLabel()
-            al.name   = label.text
-            al.index  = int(  label.attrib['index'])
-            al.x      = float(label.attrib['x'])
-            al.y      = float(label.attrib['y'])
-            al.z      = float(label.attrib['z'])
 
-            coords[i] = (al.x, al.y, al.z)
+            name  = label.text
+            index = int(  label.attrib['index'])
+            x     = float(label.attrib['x'])
+            y     = float(label.attrib['y'])
+            z     = float(label.attrib['z'])
+            al    = AtlasLabel(name, index, x, y, z)
+
+            coords[i] = (x, y, z)
 
             self.labels.append(al)
 
@@ -502,10 +530,9 @@ class AtlasDescription(object):
         # into world coordinates
         coords = transform.transform(coords, self.xforms[0])
 
-        # Update the coordinates 
+        # Update the coordinates
         # in our label objects
         for i, label in enumerate(self.labels):
-
             label.x, label.y, label.z = coords[i]
 
 
@@ -514,18 +541,18 @@ class AtlasDescription(object):
         """
         return self.atlasID == other.atlasID
 
-    
+
     def __neq__(self, other):
         """Compares the ``atlasID`` of this ``AtlasDescription`` with another.
         """
-        return self.atlasID != other.atlasID 
-    
+        return self.atlasID != other.atlasID
 
-    def __cmp__(self, other):
+
+    def __lt__(self, other):
         """Compares this ``AtlasDescription`` with another by their ``name``
         attribute.
         """
-        return cmp(self.name.lower(), other.name.lower())
+        return self.name.lower() < other.name.lower()
 
 
 class Atlas(fslimage.Image):
@@ -534,11 +561,11 @@ class Atlas(fslimage.Image):
     logic common to both.
     """
 
-    
+
     def __init__(self, atlasDesc, resolution=None, isLabel=False):
         """Initialise an ``Atlas``.
 
-        :arg atlasDesc:  The :class:`AtlasDescription` instance which 
+        :arg atlasDesc:  The :class:`AtlasDescription` instance which
                          describes the atlas.
 
         :arg resolution: Desired isotropic resolution in millimetres.
@@ -552,15 +579,17 @@ class Atlas(fslimage.Image):
         # If a reslution has not been provided,
         # choose the atlas image with the
         # highest resolution.
-        # 
+        #
         # We divide by three to get the atlas
         # image index because there are three
         # pixdim values for each atlas.
         res   = resolution
         reses = np.concatenate(atlasDesc.pixdims)
- 
-        if resolution is None: imageIdx = np.argmin(reses)               / 3
-        else:                  imageIdx = np.argmin(np.abs(reses - res)) / 3
+
+        if resolution is None: imageIdx = np.argmin(reses)
+        else:                  imageIdx = np.argmin(np.abs(reses - res))
+
+        imageIdx = imageIdx // 3
 
         if isLabel: imageFile = atlasDesc.summaryImages[imageIdx]
         else:       imageFile = atlasDesc.images[       imageIdx]
@@ -570,12 +599,12 @@ class Atlas(fslimage.Image):
         # Even though all the FSL atlases
         # are in MNI152 space, not all of
         # their sform_codes are correctly set
-        self.nibImage.get_header().set_sform(
+        self.nibImage.header.set_sform(
             None, code=constants.NIFTI_XFORM_MNI_152)
 
         self.desc = atlasDesc
 
-        
+
 class LabelAtlas(Atlas):
     """A 3D atlas which contains integer labels for each region.
 
@@ -593,32 +622,123 @@ class LabelAtlas(Atlas):
         """
         Atlas.__init__(self, atlasDesc, resolution, True)
 
-        
-    def label(self, worldLoc):
-        """Looks up and returns the label of the region at the given world
-        location, or ``None`` if the location is out of bounds.
+
+    def label(self, location, *args, **kwargs):
+        """Looks up and returns the label of the region at the given
+        location.
+
+        :arg location: Can be one of the following:
+
+                        - A sequence of three values, interpreted as
+                          atlas coordinates. In this case, :meth:`coordLabel`
+                          is called.
+
+                        - An :class:`.Image` which is interpreted as a
+                          weighted mask. In this case, :meth:`maskLabel` is
+                          called.
+
+        All other arguments are passed through to the :meth:`coordLabel` or
+        :meth:`maskLabel` methods.
+
+
+        :returns: The return value of either :meth:`coordLabel` or
+                  :meth:`maskLabel`.
         """
 
-        voxelLoc = transform.transform([worldLoc], self.worldToVoxMat)[0]
-        voxelLoc = [int(v) for v in voxelLoc.round()]
+        if isinstance(location, fslimage.Image):
+            return self.maskLabel(location, *args, **kwargs)
+        else:
+            return self.coordLabel(location, *args, **kwargs)
 
-        if voxelLoc[0] <  0             or \
-           voxelLoc[1] <  0             or \
-           voxelLoc[2] <  0             or \
-           voxelLoc[0] >= self.shape[0] or \
-           voxelLoc[1] >= self.shape[1] or \
-           voxelLoc[2] >= self.shape[2]:
+
+    def coordLabel(self, loc, voxel=False):
+        """Looks up and returns the label at the given location.
+
+        :arg loc:   A sequence of three values, interpreted as atlas
+                    coordinates. In this case, :meth:`coordLabel` is called.
+
+        :arg voxel: Defaults to ``False``. If ``True``, the ``location``
+                    is interpreted as voxel coordinates.
+
+        :returns:   The label at the given coordinates, or ``None`` if the
+                    coordinates are out of bounds.
+        """
+
+        if not voxel:
+            loc = transform.transform([loc], self.worldToVoxMat)[0]
+            loc = [int(v) for v in loc.round()]
+
+        if loc[0] <  0             or \
+           loc[1] <  0             or \
+           loc[2] <  0             or \
+           loc[0] >= self.shape[0] or \
+           loc[1] >= self.shape[1] or \
+           loc[2] >= self.shape[2]:
             return None
-        
-        val = self[voxelLoc[0], voxelLoc[1], voxelLoc[2]]
+
+        val = self[loc[0], loc[1], loc[2]]
 
         if self.desc.atlasType == 'label':
             return val
-        
+
         elif self.desc.atlasType == 'probabilistic':
             return val - 1
 
-    
+
+    def maskLabel(self, mask):
+        """Looks up and returns the proportions of all regions that are present
+        in the given ``mask``.
+
+        :arg mask: A 3D :class:`.Image`` which is interpreted as a weighted
+                   mask. If the ``mask`` shape does not match that of this
+                   ``LabelAtlas``, it is resampled using
+                   :meth:`.Image.resample`, with linear interpolation.
+
+        :returns:  A tuple containing:
+
+                     - A sequence of all labels which are present in the mask
+                     - A sequence containing the proportion, within the mask,
+                       of each present label. The proportions are returned as
+                       values between 0 and 100.
+
+        """
+
+        # Make sure that the mask has the same
+        # number of voxels as the atlas image.
+        # Use nearest neighbour interpolation
+        # for resampling, as it is most likely
+        # that the mask is binary.
+        mask     = mask.resample(self.shape[:3], dtype=np.float32, order=0)[0]
+        boolmask = mask > 0
+
+        fslimage.Image(mask, xform=self.voxToWorldMat).save('blag.nii.gz')
+
+        # Extract the labels that are in
+        # the mask, and their corresponding
+        # mask weights
+        vals      = self[boolmask]
+        weights   = mask[boolmask]
+        weightsum = weights.sum()
+        labels    = np.unique(vals)
+        props     = []
+
+        for label in labels:
+
+            # Figure out the number of all voxels
+            # in the mask with this label, weighted
+            # by the mask.
+            prop = weights[vals == label].sum()
+
+            # Normalise it to be a proportion
+            # of all voxels in the mask. We
+            # multiply by 100 because the FSL
+            # probabilistic atlases store their
+            # probabilities as percentages.
+            props.append(100 * prop / weightsum)
+
+        return labels, props
+
+
 class ProbabilisticAtlas(Atlas):
     """A 4D atlas which contains one volume for each region.
 
@@ -633,32 +753,105 @@ class ProbabilisticAtlas(Atlas):
                          the atlas.
 
         :arg resolution: Desired isotropic resolution in millimetres.
-        """ 
+        """
         Atlas.__init__(self, atlasDesc, resolution, False)
 
-        
-    def proportions(self, worldLoc):
+
+    def proportions(self, location, *args, **kwargs):
+        """Looks up and returns the proportions of of all regions at the given
+        location.
+
+        :arg location: Can be one of the following:
+
+                        - A sequence of three values, interpreted as atlas
+                          coordinates. In this case, :meth:`coordProportions`
+                          is called.
+
+                        - An :class:`.Image` which is interpreted as a
+                          weighted mask. In this case, :meth:`maskProportions`
+                          is called.
+
+        All other arguments are passed through to the :meth:`coordProportions`
+        or :meth:`maskProportions` methods.
+
+
+        :returns: The return value of either :meth:`coordProportions` or
+                  :meth:`maskProportions`.
+        """
+
+        if isinstance(location, fslimage.Image):
+            return self.maskProportions(location, *args, **kwargs)
+        else:
+            return self.coordProportions(location, *args, **kwargs)
+
+
+    def coordProportions(self, loc, voxel=False):
         """Looks up the region probabilities for the given location.
 
-        :arg worldLoc: Location in the world coordinate system.
+        :arg loc:   A sequence of three values, interpreted as atlas
+                    world or voxel coordinates.
+
+        :arg voxel: Defaults to ``False``. If ``True``, the ``loc``
+                    argument is interpreted as voxel coordinates.
 
         :returns: a list of values, one per region, which represent
                   the probability of each region for the specified
                   location. Returns an empty list if the given
                   location is out of bounds.
         """
-        voxelLoc = transform.transform([worldLoc], self.worldToVoxMat)[0]
-        voxelLoc = [int(v) for v in voxelLoc.round()]
 
-        if voxelLoc[0] <  0             or \
-           voxelLoc[1] <  0             or \
-           voxelLoc[2] <  0             or \
-           voxelLoc[0] >= self.shape[0] or \
-           voxelLoc[1] >= self.shape[1] or \
-           voxelLoc[2] >= self.shape[2]:
+        if not voxel:
+            loc = transform.transform([loc], self.worldToVoxMat)[0]
+            loc = [int(v) for v in loc.round()]
+
+        if loc[0] <  0             or \
+           loc[1] <  0             or \
+           loc[2] <  0             or \
+           loc[0] >= self.shape[0] or \
+           loc[1] >= self.shape[1] or \
+           loc[2] >= self.shape[2]:
             return []
-        
-        return self[voxelLoc[0], voxelLoc[1], voxelLoc[2], :]
+
+        return self[loc[0], loc[1], loc[2], :]
+
+
+    def maskProportions(self, mask):
+        """Looks up the probabilities of all regions in the given ``mask``.
+
+        :arg mask: A 3D :class:`.Image`` which is interpreted as a weighted
+                   mask. If the ``mask`` shape does not match that of this
+                   ``ProbabilisticAtlas``, it is resampled using
+                   :meth:`.Image.resample`, with linear interpolation.
+
+        :returns:  A tuple containing:
+
+                     - A sequence of all labels which are present in the mask
+                     - A sequence containing the proportion, within the mask,
+                       of each present label. The proportions are returned as
+                       values between 0 and 100.
+        """
+
+        labels = []
+        props  = []
+
+        # Make sure that the mask has the same
+        # number of voxels as the atlas image
+        mask      = mask.resample(self.shape[:3], dtype=np.float32, order=0)[0]
+        boolmask  = mask > 0
+        weights   = mask[boolmask]
+        weightsum = weights.sum()
+
+        for label in range(self.shape[3]):
+
+            vals = self[..., label]
+            vals = vals[boolmask] * weights
+            prop = vals.sum() / weightsum
+
+            if not np.isclose(prop, 0):
+                labels.append(label)
+                props .append(prop)
+
+        return labels, props
 
 
 registry            = AtlasRegistry()
