@@ -129,15 +129,22 @@ class FEATFSFDesign(object):
     analysis. This class is intended to be used for FEAT analyses generated
     with FSL 5.0.9 and older.
     """
-    
-    def __init__(self, featDir, settings=None):
+
+    def __init__(self, featDir, settings=None, loadVoxelwiseEVs=True):
         """Create a ``FEATFSFDesign``.
 
-        :arg featDir:      Path to the FEAT directory.
+        :arg featDir:          Path to the FEAT directory.
 
-        :arg settings:     A dictionary containing the FEAT analysis 
-                           settings from its ``design.fsf``. If not provided,
-                           is loaded via :func:`.featanalysis.loadSettings`.
+        :arg settings:         A dictionary containing the FEAT analysis
+                               settings from its ``design.fsf``. If not
+                               provided, is loaded via
+                               :func:`.featanalysis.loadSettings`.
+
+        :arg loadVoxelwiseEVs: If ``True`` (the default), image files
+                               for all voxelwise EVs are loaded. Otherwise
+                               they are not loaded, and all calls to
+                               meth:`getDesign` will contain the mean
+                               data for any voxelwise EV columns.
         """
 
         if settings is None:
@@ -146,11 +153,11 @@ class FEATFSFDesign(object):
 
         # Get the design matrix, and some
         # information about the analysis
-        designMatrix = loadDesignMat(featDir)
+        designMatrix = loadDesignMat(op.join(featDir, 'design.mat'))
         version      = float(settings['version'])
         level        = int(  settings['level'])
 
-        # Print a warning if we're 
+        # Print a warning if we're
         # using an old version of FEAT
         if version < 6:
             log.warning('Unsupported FEAT version: {}'.format(version))
@@ -174,15 +181,17 @@ class FEATFSFDesign(object):
         # so they're ready to be used by
         # the getDesign method.
         for ev in self.__evs:
-            
+
             if not isinstance(ev, (VoxelwiseEV, VoxelwiseConfoundEV)):
                 continue
 
-            # The path to some voxelwise 
-            # EVs may not be present - 
+            ev.image = None
+
+            # The path to some voxelwise
+            # EVs may not be present -
             # see the VoxelwisEV class.
-            if ev.filename is not None: ev.image = fslimage.Image(ev.filename)
-            else:                       ev.image = None
+            if loadVoxelwiseEVs and (ev.filename is not None):
+                ev.image = fslimage.Image(ev.filename)
 
 
     def getEVs(self):
@@ -190,15 +199,17 @@ class FEATFSFDesign(object):
         each column of this ``FEATFSFDesign``.
         """
         return list(self.__evs)
- 
-    
+
+
     def getDesign(self, voxel=None):
         """Returns the design matrix for the specified voxel.
 
-        :arg voxel: A tuple containing the ``(x, y, z)`` voxel coordinates
-                    of interest. If ``None`` (the default), the design
-                    matrix is returned, with any voxelwise EV columns
-                    containing the mean voxelwise EV data.
+        :arg voxel: A tuple containing the ``(x, y, z)`` voxel coordinates of
+                    interest. If ``None`` (the default), or if this
+                    ``FEATFSFDesign`` was created with
+                    ``loadVoxelwiseEVs=False``, the design matrix is returned,
+                    with any voxelwise EV columns containing the mean
+                    voxelwise EV data.
         """
 
         design = np.array(self.__design)
@@ -216,8 +227,9 @@ class FEATFSFDesign(object):
             if ev.image is None:
                 log.warning('Voxel EV image missing '
                             'for ev {}'.format(ev.index))
+                continue
 
-            design[:, ev.index] = ev.image.data[x, y, z, :]
+            design[:, ev.index] = ev.image[x, y, z, :]
 
         return design
 
@@ -242,7 +254,7 @@ class EV(object):
         self.index = index
         self.title = title
 
-        
+
 class NormalEV(EV):
     """Class representing a *normal* EV in a FEAT design matrix, i.e. one
     which has been explicitly provided by the user.
@@ -272,7 +284,7 @@ class TemporalDerivativeEV(NormalEV):
     """Class representing a temporal derivative EV, derived from a normal EV.
     """
     pass
-                          
+
 
 class BasisFunctionEV(NormalEV):
     """Class representing a basis function EV, derived from a normal EV. """
@@ -327,7 +339,7 @@ class ConfoundEV(EV):
     ============= ==========================================================
     ``confIndex`` Index of this ``ConfoundEV`` (starting from 0) in relation
                   to all other confound EVs.
-    ============= ========================================================== 
+    ============= ==========================================================
     """
     def __init__(self, index, confIndex, title):
         """Create a ``ConfoundEV``.
@@ -340,7 +352,7 @@ class ConfoundEV(EV):
         """
         EV.__init__(self, index, title)
         self.confIndex = confIndex
- 
+
 
 class MotionParameterEV(EV):
     """Class representing a motion parameter EV.
@@ -349,10 +361,10 @@ class MotionParameterEV(EV):
     addition to the :class:`EV` attributes):
 
     =============== ========================================================
-    ``motionIndex`` Index of this ``MotionParameterEV`` (starting from 0) in 
+    ``motionIndex`` Index of this ``MotionParameterEV`` (starting from 0) in
                     relation to all other motion parameter EVs.
     =============== ========================================================
-    """ 
+    """
     def __init__(self, index, motionIndex, title):
         """Create a ``MotionParameterEV``.
 
@@ -361,33 +373,33 @@ class MotionParameterEV(EV):
         :arg confIndex: Index (starting from 0) of this ``MotionParameterEV``
                         in relation to all other motion parameter EVs.
         :arg title:     Name of this ``MotionParameterEV``.
-        """ 
+        """
         EV.__init__(self, index, title)
-        self.motionIndex = motionIndex 
+        self.motionIndex = motionIndex
 
 
 class VoxelwiseConfoundEV(EV):
     """Class representing a voxelwise confound EV.
 
-    ``VoxelwiseConfoundEV`` instances contain the following attributes (in 
+    ``VoxelwiseConfoundEV`` instances contain the following attributes (in
     addition to the :class:`EV` attributes):
 
     ============ ==========================================================
-    ``voxIndex`` Index of this ``VoxelwiseConfoundEV`` (starting from 0) in 
+    ``voxIndex`` Index of this ``VoxelwiseConfoundEV`` (starting from 0) in
                  relation to all other voxelwise confound EVs.
     ``filename`` Path to the image file containing the data for this EV
-    ============ ========================================================== 
-    """ 
+    ============ ==========================================================
+    """
     def __init__(self, index, voxIndex, title, filename):
         """Create a ``Voxelwise ConfoundEV``.
 
-        :arg index:     Index (starting from 0) of this 
+        :arg index:     Index (starting from 0) of this
                         ``VoxelwiseConfoundEV`` in the design matrix.
-        :arg confIndex: Index (starting from 0) of this 
+        :arg confIndex: Index (starting from 0) of this
                         ``VoxelwiseConfoundEV`` in relation to all other
                         voxelwise confound EVs.
         :arg title:     Name of this ``VoxelwiseConfoundEV``.
-        """        
+        """
         EV.__init__(self, index, title)
         self.voxIndex = voxIndex
 
@@ -396,8 +408,8 @@ class VoxelwiseConfoundEV(EV):
         else:
             log.warning('Voxelwise confound EV file '
                         'does not exist: '.format(filename))
-            self.filename = None 
-    
+            self.filename = None
+
 
 def getFirstLevelEVs(featDir, settings, designMat):
     """Derives the EVs for the given first level FEAT analysis.
@@ -407,24 +419,24 @@ def getFirstLevelEVs(featDir, settings, designMat):
                     from its ``design.fsf`` file (see
                     :func:`.featanalysis.loadSettings`).
     :arg designMat: The FEAT design matrix (a numpy array - see
-                    :func:`.featanalysis.loadDesign`).
+                    :func:`loadDesignMat`).
 
     :returns: A list of :class:`EV` instances, one for each column in the
               design matrix.
-    """ 
+    """
 
     evs     = []
     origEVs = int(settings['evs_orig'])
 
     # First, we loop through the EVs that
     # are explicitly defined in design.fsf.
-    # This includes 
+    # This includes
     #   - normal EVs
     #   - temporal derivative EVs
     #   - basis function EVs
     #   - voxelwise EVs
     for origIdx in range(origEVs):
-            
+
         title    = settings[        'evtitle{}'  .format(origIdx + 1)]
         shape    = int(settings[    'shape{}'    .format(origIdx + 1)])
         convolve = int(settings[    'convolve{}' .format(origIdx + 1)])
@@ -433,7 +445,7 @@ def getFirstLevelEVs(featDir, settings, designMat):
 
         # Normal EV. This is just a column
         # in the design matrix, defined by
-        # the user. 
+        # the user.
         if shape != 9:
             evs.append(NormalEV(len(evs), origIdx, title))
 
@@ -451,8 +463,8 @@ def getFirstLevelEVs(featDir, settings, designMat):
             filename = op.join(
                 featDir, 'designVoxelwiseEV{}'.format(origIdx + 1))
             filename = fslimage.addExt(filename, True)
-            
-            evs.append(VoxelwiseEV(len(evs), origIdx, title, filename)) 
+
+            evs.append(VoxelwiseEV(len(evs), origIdx, title, filename))
 
         # This EV has been convolved with a set of basis
         # functions. A set of N additional EVs have been
@@ -479,7 +491,7 @@ def getFirstLevelEVs(featDir, settings, designMat):
     # have been explicilty defined, the rest of
     # the EVs in the design matrix are confounds,
     # in the following order:
-    # 
+    #
     #   1. Voxelwise confounds
     #   2. Motion parameters
     #   3. Other confounds
@@ -501,7 +513,7 @@ def getFirstLevelEVs(featDir, settings, designMat):
     # order defined in vef.dat.
     voxConfFiles = op.join(featDir, 'vef.dat')
     voxConfLocs  = op.join(featDir, 'ven.dat')
-    
+
     if op.exists(voxConfFiles) and op.exists(voxConfLocs):
 
         with open(voxConfFiles, 'rt') as vcff:
@@ -528,7 +540,7 @@ def getFirstLevelEVs(featDir, settings, designMat):
         # above, about the voxelwise
         # confound EV locations, holds
         startIdx = len(evs) + 1
-        if voxConfLocs != range(startIdx, startIdx + len(voxConfFiles)):
+        if voxConfLocs != list(range(startIdx, startIdx + len(voxConfFiles))):
             raise FSFError('Unsupported voxelwise confound ordering '
                            '({} -> {})'.format(startIdx, voxConfLocs))
 
@@ -542,14 +554,14 @@ def getFirstLevelEVs(featDir, settings, designMat):
     # Have motion parameters been added
     # as regressors to the design matrix?
     motion = int(settings['motionevs'])
-    
+
     if   motion == 1: numMotionEVs = 6
     elif motion == 2: numMotionEVs = 24
-    else:             numMotionEVs = 0 
+    else:             numMotionEVs = 0
 
     for i in range(numMotionEVs):
         evs.append(MotionParameterEV(len(evs), i, 'motion'))
-        
+
     # Last step - any columns in the design
     # matrix which have not yet been accounted
     # for are other confounds, specified by
@@ -570,10 +582,10 @@ def getHigherLevelEVs(featDir, settings, designMat):
                     from its ``design.fsf`` file (see
                     :func:`.featanalysis.loadSettings`).
     :arg designMat: The FEAT design matrix (a numpy array - see
-                    :func:`.featanalysis.loadDesign`).
+                    :func:`loadDesignMat`).
 
     :returns: A list of :class:`EV` instances, one for each column in the
-              design matrix. 
+              design matrix.
     """
 
     # TODO Maybe I can give the voxel EVs titles based on their
@@ -586,7 +598,7 @@ def getHigherLevelEVs(featDir, settings, designMat):
     #
     #   - Normal EVs
     #   - Voxelwise EVs
-    # 
+    #
     # evs_orig is the number of normal EVs
     # evs_vox is the number of voxelwise EVs
     # evs_real is the total number of EVs
@@ -611,38 +623,28 @@ def getHigherLevelEVs(featDir, settings, designMat):
     # for each voxelwise EV from its file
     # name.
     for origIdx in range(voxEVs):
-        
+
         filename = settings['evs_vox_{}'.format(origIdx + 1)]
         title    = op.basename(fslimage.removeExt(filename))
         evs.append(VoxelwiseEV(len(evs), origIdx, title, filename))
-        
+
     return evs
 
 
-def loadDesignMat(featdir):
-    """Loads the design matrix from a FEAT directory.
+def loadDesignMat(designmat):
+    """Loads the specified design matrix.
 
     Returns a ``numpy`` array containing the design matrix data, where the
     first dimension corresponds to the data points, and the second to the EVs.
 
-    :arg featdir: A FEAT directory.
+    :arg designmat: Path to the ``design.mat`` file.
     """
 
-    matrix    = None 
-    designmat = op.join(featdir, 'design.mat')
-
     log.debug('Loading FEAT design matrix from {}'.format(designmat))
 
-    with open(designmat, 'rt') as f:
+    matrix = np.loadtxt(designmat, comments='/', ndmin=2)
 
-        while True:
-            line = f.readline()
-            if line.strip() == '/Matrix':
-                break
-
-        matrix = np.loadtxt(f, ndmin=2)
-
-    if matrix is None or matrix.size == 0:
+    if matrix is None or matrix.size == 0 or len(matrix.shape) != 2:
         raise FSFError('{} does not appear to be a '
                        'valid design.mat file'.format(designmat))
 
