@@ -1,12 +1,12 @@
 #!/usr/bin/env python
-# 
+#
 # image.py - Provides the :class:`Image` class, for representing 3D/4D NIFTI
 #            images.
 #
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
 #
 """This module provides the :class:`Nifti` and :class:`Image` classes, for
-representing 3D/4D NIFTI1 and NIFTI2 images. The ``nibabel`` package is used
+representing NIFTI1 and NIFTI2 images. The ``nibabel`` package is used
 for file I/O.
 
 
@@ -23,7 +23,7 @@ and file names:
    :nosignatures:
 
    looksLikeImage
-   addExt 
+   addExt
    splitExt
    getExt
    removeExt
@@ -34,10 +34,13 @@ and file names:
 
 import                      os
 import os.path           as op
+import                      string
 import                      logging
 
-import                      six 
+import                      six
+import                      deprecation
 import numpy             as np
+import scipy.ndimage     as ndimage
 
 import nibabel           as nib
 import nibabel.fileslice as fileslice
@@ -64,25 +67,25 @@ class Nifti(notifier.Notifier):
     When a ``Nifti`` instance is created, it adds the following attributes
     to itself:
 
-    
+
     ================= ====================================================
     ``header``        The :mod:`nibabel` NIFTI1/NIFTI2/Analyze header
                       object.
-    
+
     ``shape``         A list/tuple containing the number of voxels along
                       each image dimension.
-    
-    ``pixdim``        A list/tuple containing the length of one voxel 
+
+    ``pixdim``        A list/tuple containing the length of one voxel
                       along each image dimension.
-    
+
     ``voxToWorldMat`` A 4*4 array specifying the affine transformation
                       for transforming voxel coordinates into real world
                       coordinates.
-    
+
     ``worldToVoxMat`` A 4*4 array specifying the affine transformation
                       for transforming real world coordinates into voxel
                       coordinates.
-    
+
     ``intent``        The NIFTI intent code specified in the header (or
                       :attr:`.constants.NIFTI_INTENT_NONE` for Analyze
                       images).
@@ -94,28 +97,28 @@ class Nifti(notifier.Notifier):
     writing code that should work with all three. Use the :meth:`niftiVersion`
     property if you need to know what type of image you are dealing with.
 
-    
+
     The ``shape`` attribute may not precisely match the image shape as
     reported in the NIFTI header, because trailing dimensions of size 1 are
     squeezed out. See the :meth:`__determineShape` and :meth:`mapIndices`
     methods.
 
-    
+
     **The affine transformation**
 
-    
+
     The :meth:`voxToWorldMat` and :meth:`worldToVoxMat` attributes contain
     transformation matrices for transforming between voxel and world
     coordinates. The ``Nifti`` class follows the same process as ``nibabel``
     in selecting the affine (see
     http://nipy.org/nibabel/nifti_images.html#the-nifti-affines):
 
-    
+
      1. If ``sform_code != 0`` ("unknown") use the sform affine; else
      2. If ``qform_code != 0`` ("unknown") use the qform affine; else
      3. Use the fall-back affine.
 
-    
+
     However, the *fall-back* affine used by the ``Nifti`` class differs to
     that used by ``nibabel``. In ``nibabel``, the origin (world coordinates
     (0, 0, 0)) is set to the centre of the image. Here in the ``Nifti``
@@ -127,13 +130,13 @@ class Nifti(notifier.Notifier):
     is an Analyze image). When you do so:
 
      - Only the ``sform`` of the underlying ``Nifti1Header`` object is changed
-    
+
      - The ``qform`` is not modified.
-    
+
      - If the ``sform_code`` was previously set to ``NIFTI_XFORM_UNKNOWN``,
        it is changed to ``NIFTI_XFORM_ALIGNED_ANAT``. Otherwise, the
        ``sform_code`` is not modified.
-    
+
 
     **ANALYZE support**
 
@@ -146,7 +149,7 @@ class Nifti(notifier.Notifier):
 
       - The affine will be set to a diagonal matrix with the header pixdims as
         its elements (with the X pixdim negated), and an offset specified by
-        the ANALYZE ``origin`` fields. Construction of the affine is handled 
+        the ANALYZE ``origin`` fields. Construction of the affine is handled
         by ``nibabel``.
 
       - The :meth:`niftiVersion` method will return ``0``.
@@ -158,7 +161,7 @@ class Nifti(notifier.Notifier):
     **Notification**
 
 
-    The ``Nifti`` class implements the :class:`.Notifier` interface - 
+    The ``Nifti`` class implements the :class:`.Notifier` interface -
     listeners may register to be notified on the following topics:
 
     =============== ========================================================
@@ -167,14 +170,14 @@ class Nifti(notifier.Notifier):
     =============== ========================================================
     """
 
-    
+
     def __init__(self, header):
         """Create a ``Nifti`` object.
 
-        :arg header: A :class:`nibabel.nifti1.Nifti1Header`, 
+        :arg header: A :class:`nibabel.nifti1.Nifti1Header`,
                        :class:`nibabel.nifti2.Nifti2Header`, or
                        ``nibabel.analyze.AnalyzeHeader`` to be used as the
-                       image header. 
+                       image header.
         """
 
         # Nifti2Header is a sub-class of Nifti1Header,
@@ -186,9 +189,6 @@ class Nifti(notifier.Notifier):
         header                   = header
         origShape, shape, pixdim = self.__determineShape(header)
 
-        if len(shape) < 3 or len(shape) > 4:
-            raise RuntimeError('Only 3D or 4D images are supported')
-
         voxToWorldMat = self.__determineTransform(header)
         worldToVoxMat = transform.invert(voxToWorldMat)
 
@@ -201,7 +201,7 @@ class Nifti(notifier.Notifier):
         self.__voxToWorldMat = voxToWorldMat
         self.__worldToVoxMat = worldToVoxMat
 
-    
+
     def __determineTransform(self, header):
         """Called by :meth:`__init__`. Figures out the voxel-to-world
         coordinate transformation matrix that is associated with this
@@ -242,7 +242,7 @@ class Nifti(notifier.Notifier):
         # then we can't assume that the transform
         # matrices are valid. So we fall back to a
         # pixdim scaling matrix.
-        # 
+        #
         # n.b. For images like this, nibabel returns
         # a scaling matrix where the centre voxel
         # corresponds to world location (0, 0, 0).
@@ -253,7 +253,7 @@ class Nifti(notifier.Notifier):
             voxToWorldMat = transform.scaleOffsetXform(pixdims, 0)
 
         # Otherwise we let nibabel decide
-        # which transform to use. 
+        # which transform to use.
         else:
             voxToWorldMat = np.array(header.get_best_affine())
 
@@ -261,9 +261,9 @@ class Nifti(notifier.Notifier):
 
 
     def __determineShape(self, header):
-        """This method is called by :meth:`__init__`. It figures out the actual 
-        shape of the image data, and the zooms/pixdims for each data axis. Any 
-        empty trailing dimensions are squeezed, but the returned shape is 
+        """This method is called by :meth:`__init__`. It figures out the actual
+        shape of the image data, and the zooms/pixdims for each data axis. Any
+        empty trailing dimensions are squeezed, but the returned shape is
         guaranteed to be at least 3 dimensions. Returns:
 
          - A sequence/tuple containing the image shape, as reported in the
@@ -286,10 +286,37 @@ class Nifti(notifier.Notifier):
             pixdims = header['pixdim'][1:]
 
         pixdims = pixdims[:len(shape)]
-        
+
         return origShape, shape, pixdims
 
 
+    def strval(self, key):
+        """Returns the specified NIFTI header field, converted to a python
+        string, correctly null-terminated, and with non-printable characters
+        removed.
+
+        This method is used to sanitise some NIFTI header fields. The default
+        Python behaviour for converting a sequence of bytes to a string is to
+        strip all termination characters (bytes with value of ``0x00``) from
+        the end of the sequence.
+
+        This default behaviour does not handle the case where a sequence of
+        bytes which did contain a long string is subsequently overwritten with
+        a shorter string - the short string will be terminated, but that
+        termination character will be followed by the remainder of the
+        original string.
+        """
+
+        val = self.header[key]
+
+        try:    val = bytes(val).partition(b'\0')[0]
+        except: val = bytes(val)
+
+        val = val.decode('ascii')
+
+        return ''.join([c for c in val if c in string.printable]).strip()
+
+
     @property
     def niftiVersion(self):
         """Returns the NIFTI file version:
@@ -314,7 +341,7 @@ class Nifti(notifier.Notifier):
         """Returns a tuple containing the image data shape. """
         return tuple(self.__shape)
 
-    
+
     @property
     def pixdim(self):
         """Returns a tuple containing the image pixdims (voxel sizes)."""
@@ -332,6 +359,10 @@ class Nifti(notifier.Notifier):
     def xyzUnits(self):
         """Returns the NIFTI XYZ dimension unit code. """
 
+        # analyze images have no unit field
+        if self.niftiVersion == 0:
+            return constants.NIFTI_UNITS_MM
+
         # The nibabel get_xyzt_units returns labels,
         # but we want the NIFTI codes. So we use
         # the (undocumented) nifti1.unit_codes field
@@ -345,6 +376,10 @@ class Nifti(notifier.Notifier):
     def timeUnits(self):
         """Returns the NIFTI time dimension unit code. """
 
+        # analyze images have no unit field
+        if self.niftiVersion == 0:
+            return constants.NIFTI_UNITS_SEC
+
         # See xyzUnits
         units = self.header.get_xyzt_units()[1]
         units = nib.nifti1.unit_codes[units]
@@ -413,14 +448,26 @@ class Nifti(notifier.Notifier):
         # How convenient - nibabel has a function
         # that does the dirty work for us.
         return fileslice.canonical_slicers(sliceobj, self.__origShape)
- 
-        
-    # TODO: Remove this method, and use the shape attribute directly
+
+
+    @property
+    def ndims(self):
+        """Returns the number of dimensions in this image. This number may not
+        match the number of dimensions specified in the NIFTI header, as
+        trailing dimensions of length 1 are ignored. But it is guaranteed to be
+        at least 3.
+        """
+        return len(self.__shape)
+
+
+    @deprecation.deprecated(deprecated_in='1.1.0',
+                            removed_in='2.0.0',
+                            details='Use ndims instead')
     def is4DImage(self):
         """Returns ``True`` if this image is 4D, ``False`` otherwise. """
-        return len(self.__shape) > 3 and self.__shape[3] > 1 
+        return len(self.__shape) > 3 and self.__shape[3] > 1
+
 
-    
     def getXFormCode(self, code=None):
         """This method returns the code contained in the NIFTI header,
         indicating the space to which the (transformed) image is oriented.
@@ -454,7 +501,7 @@ class Nifti(notifier.Notifier):
         # Otherwise, if the qform is
         # present, we return that.
         else:
-            
+
             sform_code = self.header['sform_code']
             qform_code = self.header['qform_code']
 
@@ -462,9 +509,9 @@ class Nifti(notifier.Notifier):
             elif qform_code != constants.NIFTI_XFORM_UNKNOWN: code = qform_code
 
         # Invalid values
-        if   code > 4: code = constants.NIFTI_XFORM_UNKNOWN
-        elif code < 0: code = constants.NIFTI_XFORM_UNKNOWN
-        
+        if code not in range(5):
+            code = constants.NIFTI_XFORM_UNKNOWN
+
         return int(code)
 
     # TODO Check what has worse performance - hashing
@@ -508,7 +555,16 @@ class Nifti(notifier.Notifier):
 
 
     @memoize.Instanceify(memoize.memoize)
+    @deprecation.deprecated(deprecated_in='1.2.0',
+                            removed_in='2.0.0',
+                            details='Use voxToScaledVoxMat instead')
     def voxelsToScaledVoxels(self):
+        """See :meth:`voxToScaledVoxMat`."""
+        return self.voxToScaledVoxMat
+
+
+    @property
+    def voxToScaledVoxMat(self):
         """Returns a transformation matrix which transforms from voxel
         coordinates into scaled voxel coordinates, with a left-right flip
         if the image appears to be stored in neurological order.
@@ -517,11 +573,17 @@ class Nifti(notifier.Notifier):
         _format_of_the_matrix_used_by_FLIRT.2C_and_how_does_it_relate_to\
         _the_transformation_parameters.3F
         """
+        return self.__voxToScaledVoxMat()
+
+
+    @memoize.Instanceify(memoize.memoize)
+    def __voxToScaledVoxMat(self):
+        """See :meth:`voxToScaledVoxMat`. """
 
         shape          = list(self.shape[ :3])
         pixdim         = list(self.pixdim[:3])
         voxToPixdimMat = np.diag(pixdim + [1.0])
-        
+
         if self.isNeurological():
             x              = (shape[0] - 1) * pixdim[0]
             flip           = transform.scaleOffsetXform([-1, 1, 1], [x, 0, 0])
@@ -530,6 +592,20 @@ class Nifti(notifier.Notifier):
         return voxToPixdimMat
 
 
+    @property
+    def scaledVoxToVoxMat(self):
+        """Returns a transformation matrix which transforms from scaled voxels
+        into voxels, the inverse of the :meth:`voxToScaledVoxMat` transform.
+        """
+        return self.__scaledVoxToVoxMat()
+
+
+    @memoize.Instanceify(memoize.memoize)
+    def __scaledVoxToVoxMat(self):
+        """See :meth:`scaledVoxToVoxMat`. """
+        return transform.invert(self.voxToScaledVoxMat)
+
+
     def sameSpace(self, other):
         """Returns ``True`` if the ``other`` image (assumed to be a
         :class:`Nifti` instance) has the same dimensions and is in the
@@ -544,7 +620,7 @@ class Nifti(notifier.Notifier):
 
 
     def getOrientation(self, axis, xform):
-        """Returns a code representing the orientation of the specified 
+        """Returns a code representing the orientation of the specified
         axis in the input coordinate system of the given transformation
         matrix.
 
@@ -559,7 +635,7 @@ class Nifti(notifier.Notifier):
 
         This method returns one of the following values, indicating the
         direction in which coordinates along the specified axis increase:
-        
+
           - :attr:`~.constants.ORIENT_L2R`:     Left to right
           - :attr:`~.constants.ORIENT_R2L`:     Right to left
           - :attr:`~.constants.ORIENT_A2P`:     Anterior to posterior
@@ -578,26 +654,26 @@ class Nifti(notifier.Notifier):
         """
 
         if self.getXFormCode() == constants.NIFTI_XFORM_UNKNOWN:
-            return constants.ORIENT_UNKNOWN 
-        
+            return constants.ORIENT_UNKNOWN
+
         code = nib.orientations.aff2axcodes(
             xform,
             ((constants.ORIENT_R2L, constants.ORIENT_L2R),
              (constants.ORIENT_A2P, constants.ORIENT_P2A),
              (constants.ORIENT_S2I, constants.ORIENT_I2S)))[axis]
 
-        return code 
+        return code
 
 
 class Image(Nifti):
-    """Class which represents a 3D/4D NIFTI image. Internally, the image is
+    """Class which represents a NIFTI image. Internally, the image is
     loaded/stored using a :mod:`nibabel.nifti1.Nifti1Image` or
     :mod:`nibabel.nifti2.Nifti2Image`, and data access managed by a
     :class:`.ImageWrapper`.
 
-    
+
     In addition to the attributes added by the :meth:`Nifti.__init__` method,
-    the following attributes/properties are present on an ``Image`` instance 
+    the following attributes/properties are present on an ``Image`` instance
     as properties (https://docs.python.org/2/library/functions.html#property):
 
 
@@ -610,36 +686,36 @@ class Image(Nifti):
                    describing its origin.
 
     ``nibImage``   A reference to the ``nibabel`` NIFTI image object.
-    
+
     ``saveState``  A boolean value which is ``True`` if this image is
                    saved to disk, ``False`` if it is in-memory, or has
                    been edited.
-    
+
     ``dataRange``  The minimum/maximum values in the image. Depending upon
                    the value of the ``calcRange`` parameter to
                    :meth:`__init__`, this may be calculated when the ``Image``
-                   is created, or may be incrementally updated as more image 
+                   is created, or may be incrementally updated as more image
                    data is loaded from disk.
     ============== ===========================================================
 
-    
+
     The ``Image`` class adds some :class:`.Notifier` topics to those which are
     already provided by the :class:`Nifti` class - listeners may register to
     be notified of changes to the above properties, by registering on the
     following _topic_ names (see the :class:`.Notifier` class documentation):
 
-    
+
     =============== ======================================================
     ``'data'``      This topic is notified whenever the image data changes
                     (via the :meth:`__setitem__` method). The indices/
                     slices of the portion of data that was modified is
                     passed to registered listeners as the notification
                     value (see :meth:`.Notifier.notify`).
-    
+
     ``'saveState'`` This topic is notified whenever the saved state of the
                     image changes (i.e. data or ``voxToWorldMat`` is
                     edited, or the image saved to disk).
-    
+
     ``'dataRange'`` This topic is notified whenever the image data range
                     is changed/adjusted.
     =============== ======================================================
@@ -654,10 +730,11 @@ class Image(Nifti):
                  loadData=True,
                  calcRange=True,
                  indexed=False,
-                 threaded=False):
+                 threaded=False,
+                 **kwargs):
         """Create an ``Image`` object with the given image data or file name.
 
-        :arg image:     A string containing the name of an image file to load, 
+        :arg image:     A string containing the name of an image file to load,
                         or a :mod:`numpy` array, or a :mod:`nibabel` image
                         object.
 
@@ -669,7 +746,7 @@ class Image(Nifti):
                         image header. Not applied to images loaded from file,
                         or existing :mod:`nibabel` images.
 
-        :arg xform:     A :math:`4\\times 4` affine transformation matrix 
+        :arg xform:     A :math:`4\\times 4` affine transformation matrix
                         which transforms voxel coordinates into real world
                         coordinates. If not provided, and a ``header`` is
                         provided, the transformation in the header is used.
@@ -682,7 +759,7 @@ class Image(Nifti):
                         from disk. In either case, the image data is
                         accessed through an :class:`.ImageWrapper` instance.
                         The data may be loaded into memory later on via the
-                        :meth:`loadData` method. 
+                        :meth:`loadData` method.
 
         :arg calcRange: If ``True`` (the default), the image range is
                         calculated immediately (vi a call to
@@ -690,7 +767,7 @@ class Image(Nifti):
                         incrementally updated as more data is read from memory
                         or disk.
 
-        :arg indexed:   If ``True``, and the file is gzipped, it is opened 
+        :arg indexed:   If ``True``, and the file is gzipped, it is opened
                         using the :mod:`indexed_gzip` package. Otherwise the
                         file is opened by ``nibabel``. Ignored if ``loadData``
                         is ``True``.
@@ -698,6 +775,9 @@ class Image(Nifti):
         :arg threaded:  If ``True``, the :class:`.ImageWrapper` will use a
                         separate thread for data range calculation. Defaults
                         to ``False``. Ignored if ``loadData`` is ``True``.
+
+        All other arguments are passed through to the ``nibabel.load`` function
+        (if it is called).
         """
 
         nibImage   = None
@@ -708,6 +788,15 @@ class Image(Nifti):
             indexed  = False
             threaded = False
 
+        # Take a copy of the header if one has
+        # been provided
+        #
+        # NOTE: Nifti extensions are copied by
+        # reference, which may cause issues in
+        # the future.
+        if header is not None:
+            header = header.copy()
+
         # The image parameter may be the name of an image file
         if isinstance(image, six.string_types):
 
@@ -727,10 +816,10 @@ class Image(Nifti):
             # Otherwise we let nibabel
             # manage the file reference(s)
             else:
-                nibImage  = nib.load(image)
-                
+                nibImage  = nib.load(image, **kwargs)
+
             dataSource = image
- 
+
         # Or a numpy array - we wrap it in a nibabel image,
         # with an identity transformation (each voxel maps
         # to 1mm^3 in real world space)
@@ -743,37 +832,54 @@ class Image(Nifti):
             # We default to NIFTI1 and not
             # NIFTI2, because the rest of
             # FSL is not yet NIFTI2 compatible.
-            nibImage = nib.nifti1.Nifti1Image(image,
-                                              xform,
-                                              header=header)
-            
+            if header is None:
+                ctr = nib.nifti1.Nifti1Image
+
+            # make sure that the data type is correct,
+            # in case this header was passed in from
+            # a different image
+            if header is not None:
+                header.set_data_dtype(image.dtype)
+
+            # But if a nibabel header has been provided,
+            # we use the corresponding image type
+            if isinstance(header, nib.nifti2.Nifti2Header):
+                ctr = nib.nifti2.Nifti2Image
+            elif isinstance(header, nib.nifti1.Nifti1Header):
+                ctr = nib.nifti1.Nifti1Image
+            elif isinstance(header, nib.analyze.AnalyzeHeader):
+                ctr = nib.analyze.AnalyzeImage
+
+            nibImage = ctr(image, xform, header=header)
+
         # otherwise, we assume that it is a nibabel image
         else:
             nibImage = image
 
-        # Figure out the name of this image, if 
+        # Figure out the name of this image, if
         # it has not beenbeen explicitly passed in
         if name is None:
-            
+
             # If this image was loaded
             # from disk, use the file name.
             if isinstance(image, six.string_types):
                 name = removeExt(op.basename(image))
-            
+
             # Or the image was created from a numpy array
             elif isinstance(image, np.ndarray):
                 name = 'Numpy array'
-            
+
             # Or image from a nibabel image
             else:
                 name = 'Nibabel image'
- 
-        Nifti.__init__(self, nibImage.get_header())
+
+        Nifti.__init__(self, nibImage.header)
 
         self.name                = name
         self.__lName             = '{}_{}'.format(id(self), self.name)
         self.__dataSource        = dataSource
         self.__fileobj           = fileobj
+        self.__threaded          = threaded
         self.__nibImage          = nibImage
         self.__saveState         = dataSource is not None
         self.__imageWrapper      = imagewrapper.ImageWrapper(self.nibImage,
@@ -789,9 +895,9 @@ class Image(Nifti):
         if calcRange:
             self.calcRange()
 
-        self.__imageWrapper.register(self.__lName, self.__dataRangeChanged) 
+        self.__imageWrapper.register(self.__lName, self.__dataRangeChanged)
+
 
-        
     def __hash__(self):
         """Returns a number which uniquely idenfities this ``Image`` instance
         (the result of ``id(self)``).
@@ -805,7 +911,7 @@ class Image(Nifti):
                                    self.name,
                                    self.dataSource)
 
-        
+
     def __repr__(self):
         """See the :meth:`__str__` method."""
         return self.__str__()
@@ -813,14 +919,21 @@ class Image(Nifti):
 
     def __del__(self):
         """Closes any open file handles, and clears some references. """
-        
+
         self.__nibImage     = None
         self.__imageWrapper = None
-        
-        if self.__fileobj is not None:
+
+        if getattr(self, '__fileobj', None) is not None:
             self.__fileobj.close()
-        
-    
+
+
+    def getImageWrapper(self):
+        """Returns the :class:`.ImageWrapper` instance used to manage
+        access to the image data.
+        """
+        return self.__imageWrapper
+
+
     @property
     def dataSource(self):
         """Returns the data source (e.g. file name) that this ``Image`` was
@@ -828,14 +941,14 @@ class Image(Nifti):
         """
         return self.__dataSource
 
-    
+
     @property
     def nibImage(self):
         """Returns a reference to the ``nibabel`` NIFTI image instance.
         """
         return self.__nibImage
 
-    
+
     @property
     def saveState(self):
         """Returns ``True`` if this ``Image`` has been saved to disk, ``False``
@@ -843,7 +956,7 @@ class Image(Nifti):
         """
         return self.__saveState
 
-    
+
     @property
     def dataRange(self):
         """Returns the image data range as a  ``(min, max)`` tuple. If the
@@ -867,17 +980,17 @@ class Image(Nifti):
 
         return drange
 
-    
+
     @property
     def dtype(self):
         """Returns the ``numpy`` data type of the image data. """
-        
+
         # Get the data type from the
         # first voxel in the image
         coords = [0] * len(self.__nibImage.shape)
         return self.__nibImage.dataobj[tuple(coords)].dtype
 
-    
+
     @Nifti.voxToWorldMat.setter
     def voxToWorldMat(self, xform):
         """Overrides the :meth:`Nifti.voxToWorldMat` property setter.
@@ -886,12 +999,12 @@ class Image(Nifti):
         updates the header, and this implementation makes sure the
         image is also updated.
         """
-        
+
         Nifti.voxToWorldMat.fset(self, xform)
-        
+
         xform =     self.voxToWorldMat
         code  = int(self.header['sform_code'])
-        
+
         self.__nibImage.set_sform(xform, code)
 
 
@@ -917,7 +1030,7 @@ class Image(Nifti):
 
         :arg sizethres: If not ``None``, specifies an image size threshold
                         (total number of bytes). If the number of bytes in
-                        the image is greater than this threshold, the range 
+                        the image is greater than this threshold, the range
                         is calculated on a sample (the first volume for a
                         4D image, or slice for a 3D image).
         """
@@ -935,16 +1048,15 @@ class Image(Nifti):
             log.debug('{}: Forcing calculation of full '
                       'data range'.format(self.name))
             self.__imageWrapper[:]
-            
+
         else:
             log.debug('{}: Calculating data range '
                       'from sample'.format(self.name))
 
             # Otherwise if the number of values in the
-            # image is bigger than the size threshold, 
+            # image is bigger than the size threshold,
             # we'll calculate the range from a sample:
-            if len(self.shape) == 3: self.__imageWrapper[:, :, 0]
-            else:                    self.__imageWrapper[:, :, :, 0]
+            self.__imageWrapper[..., 0]
 
 
     def loadData(self):
@@ -998,14 +1110,114 @@ class Image(Nifti):
             nib.save(self.__nibImage, filename)
             self.__fileobj.close()
             self.__nibImage, self.__fileobj = loadIndexedImageFile(filename)
-            self.header = self.__nibImage.get_header()
+            self.header = self.__nibImage.header
+
+            # We have to create a new ImageWrapper
+            # instance too, as we have just destroyed
+            # the nibabel image we gave to the last
+            # one.
+            self.__imageWrapper.deregister(self.__lName)
+            self.__imageWrapper = imagewrapper.ImageWrapper(
+                self.nibImage,
+                self.name,
+                loadData=False,
+                dataRange=self.dataRange,
+                threaded=self.__threaded)
+            self.__imageWrapper.register(self.__lName, self.__dataRangeChanged)
 
         self.__dataSource = filename
         self.__saveState  = True
-        
+
         self.notify(topic='saveState')
 
 
+    def resample(self,
+                 newShape,
+                 sliceobj=None,
+                 dtype=None,
+                 order=1,
+                 smooth=True):
+        """Returns a copy of the data in this ``Image``, resampled to the
+        specified ``shape``.
+
+        :arg newShape: Desired shape. May containg floating point values,
+                       in which case the resampled image will have shape
+                       ``round(newShape)``, but the voxel sizes will
+                       have scales ``self.shape / newShape``.
+
+        :arg sliceobj: Slice into this ``Image``. If ``None``, the whole
+                       image is resampled, and it is assumed that it has the
+                       same number of dimensions as  ``shape``.
+
+        :arg dtype:    ``numpy`` data type of the resampled data. If ``None``,
+                       the :meth:`dtype` of this ``Image`` is used.
+
+        :arg order:    Spline interpolation order, passed through to the
+                       ``scipy.ndimage.affine_transform`` function - ``0``
+                       corresponds to nearest neighbour interpolation, ``1``
+                       (the default) to linear interpolation, and ``3`` to
+                       cubic interpolation.
+
+        :arg smooth:   If ``True`` (the default), the data is smoothed before
+                       being resampled, but only along axes which are being
+                       down-sampled (i.e. where
+                       ``newShape[i] < self.shape[i]``).
+
+        :returns: A tuple containing:
+
+                   - A ``numpy`` array of shape ``shape``, containing an
+                     interpolated copy of the data in this ``Image``.
+
+                   - A ``numpy`` array of shape ``(4, 4)``, containing the
+                     adjusted voxel-to-world transformation for the resampled
+                     data.
+        """
+
+        if sliceobj is None: sliceobj = slice(None)
+        if dtype    is None: dtype    = self.dtype
+
+        data = self[sliceobj]
+        data = np.array(data, dtype=dtype, copy=False)
+
+        oldShape = np.array(data.shape, dtype=np.float)
+        newShape = np.array(newShape,   dtype=np.float)
+
+        if not np.all(np.isclose(oldShape, newShape)):
+
+            ratio    = oldShape / newShape
+            newShape = np.array(np.round(newShape), dtype=np.int)
+            scale    = transform.scaleOffsetXform(ratio, 0)
+
+            # If interpolating and smoothing, we apply a
+            # gaussian filter along axes with a resampling
+            # ratio greater than 1.1. We do this so that
+            # interpolation has an effect when down-sampling
+            # to a resolution where the voxel centres are
+            # aligned (as otherwise any interpolation regime
+            # will be equivalent to nearest neighbour). This
+            # more-or-less mimics the behaviour of FLIRT.
+            if order > 0 and smooth:
+                sigma                = np.array(ratio)
+                sigma[ratio <  1.1]  = 0
+                sigma[ratio >= 1.1] *= 0.425
+                data = ndimage.gaussian_filter(data, sigma)
+
+            data = ndimage.affine_transform(data,
+                                            scale[:3, :3],
+                                            output_shape=newShape,
+                                            order=order)
+
+            # Construct an affine transform which
+            # puts the resampled image into the
+            # same world coordinate system as this
+            # image.
+            xform = transform.concat(self.voxToWorldMat, scale)
+        else:
+            xform = self.voxToWorldMat
+
+        return data, xform
+
+
     def __getitem__(self, sliceobj):
         """Access the image data with the specified ``sliceobj``.
 
@@ -1023,9 +1235,10 @@ class Image(Nifti):
         :arg sliceobj: Something which can slice the image data.
         :arg values:   New image data.
 
-        .. note:: Modifying image data will force the entire image to be 
+        .. note:: Modifying image data will force the entire image to be
                   loaded into memory if it has not already been loaded.
         """
+        values = np.array(values)
 
         log.debug('{}: __setitem__ [{} = {}]'.format(self.name,
                                                      sliceobj,
@@ -1046,7 +1259,7 @@ class Image(Nifti):
                 self.notify(topic='saveState')
 
             if not np.all(np.isclose(oldRange, newRange)):
-                self.notify(topic='dataRange') 
+                self.notify(topic='dataRange')
 
 
 ALLOWED_EXTENSIONS = ['.nii.gz', '.nii', '.img', '.hdr', '.img.gz', '.hdr.gz']
@@ -1087,7 +1300,7 @@ def looksLikeImage(filename, allowedExts=None):
               ``myfile``).
 
     :arg filename:    The file name to test.
-    
+
     :arg allowedExts: A list of strings containing the allowed file
                       extensions - defaults to :attr:`ALLOWED_EXTENSIONS`.
     """
@@ -1145,7 +1358,7 @@ def defaultExt():
         'NIFTI_PAIR' : '.img',
         'NIFTI_GZ'   : '.nii.gz',
     }
-    
+
     outputType = os.environ.get('FSLOUTPUTTYPE', 'NIFTI_GZ')
 
     return options.get(outputType, '.nii.gz')
@@ -1160,7 +1373,7 @@ def loadIndexedImageFile(filename):
 
     import                 threading
     import indexed_gzip as igzip
-    
+
     log.debug('Loading {} using indexed gzip'.format(filename))
 
     # guessed_image_type returns a
@@ -1201,12 +1414,12 @@ def read_segments(fileobj, segments, n_bytes):
     """
 
     from mmap import mmap
-    
+
     try:
         # fileobj is a nibabel.openers.ImageOpener - the
         # actual file is available via the fobj attribute
         lock = getattr(fileobj.fobj, '_arrayproxy_lock')
-        
+
     except:
         return fileslice.orig_read_segments(fileobj, segments, n_bytes)
 
@@ -1223,27 +1436,30 @@ def read_segments(fileobj, segments, n_bytes):
             bytes = fileobj.read(length)
         finally:
             lock.release()
-            
+
         if len(bytes) != n_bytes:
             raise ValueError("Whoops, not enough data in file")
         return bytes
-    
+
     # More than one segment
     bytes = mmap(-1, n_bytes)
     for offset, length in segments:
 
         lock.acquire()
-        try: 
+        try:
             fileobj.seek(offset)
             bytes.write(fileobj.read(length))
         finally:
             lock.release()
-            
+
     if bytes.tell() != n_bytes:
         raise ValueError("Oh dear, n_bytes does not look right")
-    return bytes 
+    return bytes
 
 
-# Monkey-patch the above implementation into nibabel
-fileslice.orig_read_segments = fileslice.read_segments
-fileslice.read_segments      = read_segments
+# Monkey-patch the above implementation into
+# nibabel. FSLeyes requires at least 2.1.0 -
+# newer versions do not need to be patched.
+if nib.__version__ == '2.1.0':
+    fileslice.orig_read_segments = fileslice.read_segments
+    fileslice.read_segments      = read_segments
