@@ -7,11 +7,14 @@
 
 from __future__ import print_function
 
+
+
 import               os
 import os.path    as op
 import               shutil
-import subprocess as sp 
+import subprocess as sp
 import               tempfile
+import               logging
 
 import numpy   as np
 import nibabel as nib
@@ -26,33 +29,25 @@ import fsl.scripts.imcp as imcp_script
 import fsl.scripts.immv as immv_script
 import fsl.data.image   as fslimage
 
+from . import make_random_image
+from . import make_dummy_file
+from . import looks_like_image
+from . import cleardir
+
+
 real_print = print
-def print(*a, **kwa):
+
+def print(*args, **kwargs):
     pass
 
 
-def createImageFile(filename):
-
-    data = np.random.random((10, 10, 10))
-
-    # Image file
-    try:
-        img  = nib.Nifti1Image(data, np.eye(4))
-
-        nib.save(img, filename)
-
-        return hash(data.tobytes())
-
-    # Non-image file
-    except:
-        contents = '{}\n'.format(op.basename(filename))
-        with open(filename, 'wt') as f:
-            f.write(contents)
-
-        return hash(contents)
+def makeImage(filename):
+    return hash(make_random_image(filename).get_data().tobytes())
 
 
-def checkImageFile(filename, datahash):
+def checkImageHash(filename, datahash):
+    """Checks that the given NIFTI image matches the given hash.
+    """
 
     img = nib.load(filename)
     assert hash(img.get_data().tobytes()) == datahash
@@ -113,29 +108,22 @@ def checkFilesToExpect(files, outdir, outputType, datahashes):
         else:
             h = datahashes[op.basename(f)]
 
-        checkImageFile(f, h)
-
-def cleardir(dir):
-
-    for f in os.listdir(dir):
-        f = op.join(dir, f)
-        if   op.isfile(f): os.remove(f)
-        elif op.isdir(f):  shutil.rmtree(f)
+        checkImageHash(f, h)
 
 
 def test_imcp_script_shouldPass(move=False):
-    
+
 
     # The imcp/immv scripts should honour the
-    # FSLOUTPUTTYPE env var. If it is unset 
+    # FSLOUTPUTTYPE env var. If it is unset
     # or invalid - '' in this case), they
     # should produce .nii.gz
     outputTypes = ['NIFTI', 'NIFTI_PAIR', 'NIFTI_GZ', '']
- 
+
 
     # Test tuples have the following structure (each
     # item is a string which will be split on spaces):
-    # 
+    #
     #   (files_to_create, imcp_args, files_to_expect)
     #
     # The files_to_expect is a list of
@@ -152,26 +140,26 @@ def test_imcp_script_shouldPass(move=False):
         ('a.nii', 'a     b.nii',    'b'),
         ('a.nii', 'a.nii b.nii',    'b'),
         ('a.nii', 'a     .',        'a'),
-        ('a.nii', 'a.nii .',        'a'), 
+        ('a.nii', 'a.nii .',        'a'),
         ('a.nii', 'a     b.hdr',    'b'),
         ('a.nii', 'a     b.img',    'b'),
         ('a.nii', 'a     b.nii.gz', 'b'),
         ('a.nii', 'a.nii b.hdr',    'b'),
         ('a.nii', 'a.nii b.img',    'b'),
-        ('a.nii', 'a.nii b.nii.gz', 'b'), 
-        
+        ('a.nii', 'a.nii b.nii.gz', 'b'),
+
         ('a.nii.gz', 'a        b',        'b'),
         ('a.nii.gz', 'a.nii.gz b',        'b'),
         ('a.nii.gz', 'a        b.nii.gz', 'b'),
         ('a.nii.gz', 'a.nii.gz b.nii.gz', 'b'),
         ('a.nii.gz', 'a        .',        'a'),
-        ('a.nii.gz', 'a.nii.gz .',        'a'), 
+        ('a.nii.gz', 'a.nii.gz .',        'a'),
         ('a.nii.gz', 'a        b.hdr',    'b'),
         ('a.nii.gz', 'a        b.img',    'b'),
         ('a.nii.gz', 'a        b.nii',    'b'),
         ('a.nii.gz', 'a.nii.gz b.hdr',    'b'),
         ('a.nii.gz', 'a.nii.gz b.img',    'b'),
-        ('a.nii.gz', 'a.nii.gz b.nii',    'b'), 
+        ('a.nii.gz', 'a.nii.gz b.nii',    'b'),
 
         ('a.img', 'a        b',        'b'),
         ('a.img', 'a        b.img',    'b'),
@@ -184,8 +172,8 @@ def test_imcp_script_shouldPass(move=False):
         ('a.img', 'a.hdr    b',        'b'),
         ('a.img', 'a.hdr    b.img',    'b'),
         ('a.img', 'a.hdr    b.hdr',    'b'),
-        ('a.img', 'a.hdr    .',        'a'), 
-        
+        ('a.img', 'a.hdr    .',        'a'),
+
         ('a.img', 'a        b.nii',    'b'),
         ('a.img', 'a        b.nii.gz', 'b'),
         ('a.img', 'a        .',        'a'),
@@ -209,7 +197,7 @@ def test_imcp_script_shouldPass(move=False):
         ('a.img b.img', 'a.img b.hdr .',   'a b'),
         ('a.img b.img', 'a.hdr b     .',   'a b'),
         ('a.img b.img', 'a.hdr b.img .',   'a b'),
-        ('a.img b.img', 'a.hdr b.hdr .',   'a b'), 
+        ('a.img b.img', 'a.hdr b.hdr .',   'a b'),
 
         ('a.nii.gz b.nii.gz', 'a        b        .',   'a b'),
         ('a.nii.gz b.nii.gz', 'a        b.nii.gz .',   'a b'),
@@ -244,7 +232,7 @@ def test_imcp_script_shouldPass(move=False):
         ('a.img',       'a.hdr a                 .', 'a'),
         ('a.img',       'a.hdr a.img             .', 'a'),
         ('a.img',       'a.hdr a.hdr             .', 'a'),
-        
+
         ('a.img b.img', 'a     a     b     b     .', 'a b'),
         ('a.img b.img', 'a     a     b     b.img .', 'a b'),
         ('a.img b.img', 'a     a     b     b.hdr .', 'a b'),
@@ -280,7 +268,7 @@ def test_imcp_script_shouldPass(move=False):
         ('a.img b.img', 'a.img a.hdr b     b.hdr .', 'a b'),
         ('a.img b.img', 'a.img a.hdr b.img b     .', 'a b'),
         ('a.img b.img', 'a.img a.hdr b.img b.img .', 'a b'),
-        ('a.img b.img', 'a.img a.hdr b.img b.hdr .', 'a b'),         
+        ('a.img b.img', 'a.img a.hdr b.img b.hdr .', 'a b'),
         ('a.img b.img', 'a.hdr a     b     b     .', 'a b'),
         ('a.img b.img', 'a.hdr a     b     b.img .', 'a b'),
         ('a.img b.img', 'a.hdr a     b     b.hdr .', 'a b'),
@@ -315,18 +303,18 @@ def test_imcp_script_shouldPass(move=False):
         ('a.img    a.nii.gz a.nii   ', 'a.img    a.nii.gz a.nii    .', 'a'),
         ('a.img    a.nii    a.nii.gz', 'a.img    a.nii    a.nii.gz .', 'a'),
         ('a.nii.gz a.img    a.nii   ', 'a.nii.gz a.img    a.nii    .', 'a'),
-        ('a.nii.gz a.nii    a.img   ', 'a.nii.gz a.nii    a.img    .', 'a'), 
+        ('a.nii.gz a.nii    a.img   ', 'a.nii.gz a.nii    a.img    .', 'a'),
     ]
 
     indir  = tempfile.mkdtemp()
     outdir = tempfile.mkdtemp()
-    
+
     try:
 
         for outputType in outputTypes:
-            
+
             os.environ['FSLOUTPUTTYPE'] = outputType
-            
+
             for files_to_create, imcp_args, files_to_expect in tests:
 
                 imageHashes = []
@@ -337,7 +325,7 @@ def test_imcp_script_shouldPass(move=False):
                 print('files_to_expect: ', files_to_expect)
 
                 for i, fname in enumerate(files_to_create.split()):
-                    imageHashes.append(createImageFile(op.join(indir, fname)))
+                    imageHashes.append(makeImage(op.join(indir, fname)))
 
                 imcp_args = imcp_args.split()
 
@@ -351,7 +339,7 @@ def test_imcp_script_shouldPass(move=False):
                 else:    imcp_script.main(imcp_args)
 
                 print('indir after:     ', os.listdir(indir))
-                print('outdir after:    ', os.listdir(outdir)) 
+                print('outdir after:    ', os.listdir(outdir))
 
                 checkFilesToExpect(
                     files_to_expect, outdir, outputType, imageHashes)
@@ -364,11 +352,11 @@ def test_imcp_script_shouldPass(move=False):
                 cleardir(indir)
                 cleardir(outdir)
 
-        
+
     finally:
         shutil.rmtree(indir)
         shutil.rmtree(outdir)
-        
+
 
 
 def test_imcp_script_shouldFail(move=False):
@@ -379,11 +367,11 @@ def test_imcp_script_shouldFail(move=False):
     # - ambiguous inputs
     # - input is incomplete pair (e.g. .hdr
     #   without corresponding .img)
-    
+
     # a.img
     # a.hdr
     # a.nii
-    # 
+    #
     # FAIL: imcp a           dest
 
     # (files_to_create, imcp_args[, preproc])
@@ -415,7 +403,7 @@ def test_imcp_script_shouldFail(move=False):
         ('a.img', 'a.hdr b', 'rm indir/a.hdr'),
         ('a.img', 'a     b', 'rm indir/a.img'),
         ('a.img', 'a.img b', 'rm indir/a.img'),
-        ('a.img', 'a.hdr b', 'rm indir/a.img'), 
+        ('a.img', 'a.hdr b', 'rm indir/a.img'),
     ]
 
     if move:
@@ -428,13 +416,13 @@ def test_imcp_script_shouldFail(move=False):
 
     indir  = tempfile.mkdtemp()
     outdir = tempfile.mkdtemp()
-    
+
     try:
         for test in tests:
 
             files_to_create = test[0]
             imcp_args       = test[1]
-            
+
             if len(test) == 3: preproc = test[2]
             else:              preproc = None
 
@@ -442,7 +430,7 @@ def test_imcp_script_shouldFail(move=False):
             imcp_args       = imcp_args      .split()
 
             for fname in files_to_create:
-                createImageFile(op.join(indir, fname))
+                makeImage(op.join(indir, fname))
 
             imcp_args[:-1] = [op.join(indir, a) for a in imcp_args[:-1]]
             imcp_args[ -1] =  op.join(outdir, imcp_args[-1])
@@ -473,9 +461,9 @@ def test_imcp_script_shouldFail(move=False):
 def test_immv_script_shouldPass():
     test_imcp_script_shouldPass(move=True)
 
-    
+
 def test_immv_script_shouldFail():
-    test_imcp_script_shouldFail(move=True) 
+    test_imcp_script_shouldFail(move=True)
 
 
 def test_imcp_shouldPass(move=False):
@@ -491,7 +479,7 @@ def test_imcp_shouldPass(move=False):
     #      imgp src.img dest     -> dest.nii.gz
     #      imgp src.img dest.nii -> dest.nii.gz
 
-    # 
+    #
     # (files_to_create,
     #    [( imcp_args, files_which_should_exist),
     #     ...
@@ -542,9 +530,9 @@ def test_imcp_shouldPass(move=False):
             ('file.nii file.nii', 'file.nii'),
 
             # TODO both img/nii files
-        ]),        
-                
-        
+        ]),
+
+
         ('file.nii', [
             ('file     file',     'file.nii'),
             ('file     file.nii', 'file.nii'),
@@ -563,7 +551,7 @@ def test_imcp_shouldPass(move=False):
             ('file.nii.gz .',           'file.nii.gz'),
         ]),
 
-        
+
         ('file.nii file.blob', [
             ('file     file',     'file.nii'),
             ('file     file.nii', 'file.nii'),
@@ -571,8 +559,8 @@ def test_imcp_shouldPass(move=False):
             ('file.nii file',     'file.nii'),
             ('file.nii file.nii', 'file.nii'),
             ('file.nii .',        'file.nii'),
-        ]), 
-        
+        ]),
+
 
         ('file.nii file.nii.gz', [
             ('file.nii    file',        'file.nii'),
@@ -595,11 +583,11 @@ def test_imcp_shouldPass(move=False):
 
 
         ('001.img 002.img 003.img', [
-            
+
             ('001     002     003     .', '001.img 002.img 003.img'),
             ('001.img 002.img 003.img .', '001.img 002.img 003.img'),
             ('001.hdr 002.hdr 003.hdr .', '001.img 002.img 003.img'),
-                                               
+
             ('001.img 002     003     .', '001.img 002.img 003.img'),
             ('001.hdr 002     003     .', '001.img 002.img 003.img'),
 
@@ -609,13 +597,13 @@ def test_imcp_shouldPass(move=False):
             ('001     003             .', '001.img 003.img'),
             ('001.img 003.img         .', '001.img 003.img'),
             ('001.hdr 003.hdr         .', '001.img 003.img'),
-                                      
+
             ('001.img 003             .', '001.img 003.img'),
             ('001.hdr 003             .', '001.img 003.img'),
 
             ('001.img 003.img         .', '001.img 003.img'),
-            ('001.hdr 003.hdr         .', '001.img 003.img'), 
-        ]),  
+            ('001.hdr 003.hdr         .', '001.img 003.img'),
+        ]),
     ]
 
 
@@ -627,7 +615,7 @@ def test_imcp_shouldPass(move=False):
         for files_to_create, tests in shouldPass:
 
             files_to_create = files_to_create.split()
-            
+
             for imcp_args, should_exist in tests:
 
                 should_exist    = should_exist.split()
@@ -637,7 +625,10 @@ def test_imcp_shouldPass(move=False):
 
                 hashes = {}
                 for fn in files_to_create:
-                    hashes[fn] = createImageFile(op.join(indir, fn))
+                    if looks_like_image(fn):
+                        hashes[fn] = makeImage(op.join(indir, fn))
+                    else:
+                        hashes[fn] = make_dummy_file(op.join(indir, fn))
 
                 print()
                 print('files_to_create: ', files_to_create)
@@ -651,7 +642,7 @@ def test_imcp_shouldPass(move=False):
                     print('  src: {}'.format(src))
 
                     src = op.join(indir, src)
-                    
+
                     if move: imcp.immv(src, op.join(outdir, imcp_dest), overwrite=True)
                     else:    imcp.imcp(src, op.join(outdir, imcp_dest), overwrite=True)
 
@@ -670,15 +661,15 @@ def test_imcp_shouldPass(move=False):
                 if move:
                     for f in should_exist:
                          assert not op.exists(op.join(indir, f))
-                         
+
                 for f in os.listdir(indir):
                     try:    os.remove(op.join(indir,  f))
                     except: pass
-                        
+
                 for f in os.listdir(outdir):
                     os.remove(op.join(outdir, f))
-        
-        
+
+
     finally:
         shutil.rmtree(indir)
         shutil.rmtree(outdir)
